@@ -7,7 +7,8 @@
  *
  */
 
-/* Copyright (C) 2010-2019 Jose E. Marchesi */
+/* Copyright (C) 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018,
+ * 2019, 2020 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -115,7 +116,7 @@ Convert csv data into rec data.\n"), stdout);
      no-wrap */
   fputs (_("\
   -t, --type=TYPE                     type name for the converted records; if this\n\
-                                        parameter is ommited then no type is used.\n\
+                                        parameter is omitted then no type is used.\n\
   -s, --strict                        be strict parsing the csv file.\n\
   -e, --omit-empty                    omit empty fields.\n"), stdout);
 
