@@ -7,7 +7,8 @@
  *
  */
 
-/* Copyright (C) 2011-2019 Jose E. Marchesi */
+/* Copyright (C) 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
+ * 2020 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -61,6 +62,7 @@ char              rec2csv_delim          = ',';
 enum
   {
     COMMON_ARGS,
+    DELIM_ARG,
     RECORD_TYPE_ARG,
     SORT_ARG
   };
@@ -68,6 +70,7 @@ enum
 static const struct option GNU_longOptions[] =
   {
     COMMON_LONG_ARGS,
+    {"delim", required_argument, NULL, DELIM_ARG},
     {"type", required_argument, NULL, RECORD_TYPE_ARG},
     {"sort", required_argument, NULL, SORT_ARG},
     {NULL, 0, NULL, 0}
@@ -123,12 +126,13 @@ rec2csv_parse_args (int argc,
       switch (c)
         {
           COMMON_ARGS_CASES
-        case RECORD_TYPE_ARG:
+        case DELIM_ARG:
         case 'd':
           {
             rec2csv_delim = optarg[0];
             break;
           }
+        case RECORD_TYPE_ARG:
         case 't':
           {
             rec2csv_record_type = xstrdup (optarg);
